@@ -19,7 +19,6 @@
 #include "EphyWrapper.h"
 #include "GlobalHistory.h"
 #include "ProgressListener.h"
-#include "PrintProgressListener.h"
 #include "ephy-embed.h"
 #include "ephy-string.h"
 
@@ -37,11 +36,9 @@
 #include "nsIHistoryEntry.h"
 #include "nsIWebBrowserPrint.h"
 #include "nsIURI.h"
-#include "nsIPresShell.h"
 #include "nsIMarkupDocumentViewer.h"
 #include "nsIComponentManager.h"
 #include "nsIScriptGlobalObject.h"
-#include "nsIScriptContext.h"
 #include "nsIDOMWindowInternal.h"
 #include "nsIInterfaceRequestor.h"
 #include "nsIFocusController.h"
@@ -67,12 +64,9 @@
 #include "nsIDOMHTMLDocument.h"
 #include "nsIDOMHTMLCollection.h"
 #include "nsIDOMHTMLElement.h"
-#include "caps/nsIPrincipal.h"
 #include "nsIDeviceContext.h"
 #include "nsIPresContext.h"
 #include "ContentHandler.h"
-#include "nsITypeAheadFind.h"
-#include "nsSupportsPrimitives.h"
 #include "EphyEventListener.h"
 
 EphyWrapper::EphyWrapper ()
@@ -198,8 +192,7 @@ nsresult EphyWrapper::Print (nsIPrintSettings *options, PRBool preview)
 
 	if (!preview)
 	{
-		GPrintListener *listener = new GPrintListener();
-		result = print->Print (options, listener);
+		result = print->Print (options, nsnull);
 	}
 	else
 	{
@@ -218,7 +211,7 @@ nsresult EphyWrapper::PrintPreviewClose (void)
 	if (NS_FAILED(rv) || !print) return NS_ERROR_FAILURE;
 
 	rv = print->GetDoingPrintPreview(&isPreview);
-	if (isPreview == PR_TRUE)
+	if (NS_SUCCEEDED (rv) && isPreview == PR_TRUE)
 	{
 		rv = print->ExitPrintPreview();
 	}
@@ -360,6 +353,7 @@ nsresult EphyWrapper::SetZoomOnDocshell (float aZoom, nsIDocShell *DocShell)
 					
 	nsCOMPtr<nsIDeviceContext> DeviceContext;
 	result = PresContext->GetDeviceContext (getter_AddRefs(DeviceContext));
+	if (NS_FAILED(result) || !DeviceContext) return NS_ERROR_FAILURE;
 
 	return DeviceContext->SetTextZoom (aZoom);
 }
@@ -502,50 +496,25 @@ nsresult EphyWrapper::GetSHUrlAtIndex (PRInt32 index, nsCString &url)
 	return NS_OK;
 }
 
-nsresult EphyWrapper::Find (const PRUnichar *search_string, 
-			    PRBool interactive,
-			    PRBool matchcase, PRBool search_backwards,
-			    PRBool search_wrap_around,
-			    PRBool search_for_entire_word,
-			    PRBool search_in_frames,
-			    PRBool *didFind)
+nsresult EphyWrapper::FindSetProperties (const PRUnichar *search_string,
+			                 PRBool case_sensitive,
+					 PRBool wrap_around)
 {
-	if (!interactive)
-	{
-		nsresult rv;
-		nsCOMPtr<nsITypeAheadFind> tAFinder
-			(do_GetService(NS_TYPEAHEADFIND_CONTRACTID, &rv));
-		if (NS_SUCCEEDED(rv))
-		{
-			nsCOMPtr<nsIDOMWindow> aFocusedWindow;
-			rv = GetFocusedDOMWindow(getter_AddRefs(aFocusedWindow));
-			if (NS_SUCCEEDED(rv))
-			{
-				nsSupportsInterfacePointerImpl windowPtr;
-				windowPtr.SetData(aFocusedWindow);
-
-				tAFinder->FindNext(search_backwards, &windowPtr);
-
-				nsCOMPtr<nsISupports> retValue;
-				rv = windowPtr.GetData(getter_AddRefs(retValue));
-				if (NS_SUCCEEDED(rv) && !retValue)
-				{
-					*didFind = PR_TRUE;
-					return NS_OK;
-				}
-			}
-		}
-
-	}
-
 	nsCOMPtr<nsIWebBrowserFind> finder (do_GetInterface(mWebBrowser));
 	
 	finder->SetSearchString (search_string);
-	finder->SetFindBackwards (search_backwards);
-	finder->SetWrapFind (search_wrap_around);
-	finder->SetEntireWord (search_for_entire_word);
-	finder->SetMatchCase (matchcase);
-	finder->SetSearchFrames (search_in_frames);
+	finder->SetMatchCase (case_sensitive);
+
+	return NS_OK;
+}
+
+nsresult EphyWrapper::Find (PRBool backwards,
+			    PRBool *didFind)
+{
+	nsCOMPtr<nsIWebBrowserFind> finder (do_GetInterface(mWebBrowser));
+	
+	finder->SetFindBackwards (backwards);
+
 	return finder->FindNext(didFind);
 }
 
@@ -571,21 +540,6 @@ nsresult EphyWrapper::GetWebNavigation(nsIWebNavigation **aWebNavigation)
 	return NS_OK;
 }
 
-nsresult EphyWrapper::ReloadDocument ()
-{
-	nsresult result;
-
-	nsCOMPtr<nsIWebNavigation> wn;
-	result = GetWebNavigation(getter_AddRefs(wn));
-	if (!wn || !NS_SUCCEEDED (result)) return NS_ERROR_FAILURE;
-
-	result = wn->Reload (nsIWebNavigation::LOAD_FLAGS_BYPASS_CACHE | 
-			     nsIWebNavigation::LOAD_FLAGS_BYPASS_PROXY);
-	if (!NS_SUCCEEDED (result)) return NS_ERROR_FAILURE;
-
-	return NS_OK;
-}
-
 nsresult EphyWrapper::LoadDocument(nsISupports *aPageDescriptor,
 				     PRUint32 aDisplayType)
 {
@@ -714,7 +668,7 @@ nsresult  EphyWrapper::CopyHistoryTo (EphyWrapper *dest)
 	return NS_OK;
 }
 
-nsresult EphyWrapper::ForceCharacterSet (const char *charset) 
+nsresult EphyWrapper::ForceEncoding (const char *encoding) 
 {
 	nsresult result;
 
@@ -730,7 +684,11 @@ nsresult EphyWrapper::ForceCharacterSet (const char *charset)
 								  &result);
 	if (NS_FAILED(result) || !mdv) return NS_ERROR_FAILURE;
 
-	result = mdv->SetForceCharacterSet (NS_ConvertUTF8toUCS2(charset).get());
+#if MOZILLA_SNAPSHOT > 8
+	result = mdv->SetForceCharacterSet (nsDependentCString(encoding));
+#else
+	result = mdv->SetForceCharacterSet (NS_ConvertUTF8toUCS2(encoding).get());
+#endif
 
 	return result;
 }
