@@ -23,11 +23,14 @@
 #include "eggmarshalers.h"
 #include "eggtoolbar.h"
 #include <gtk/gtkseparatormenuitem.h>
+#include <string.h>
 
 #ifndef _
 #  define _(s) (s)
 #endif
 
+#define MENU_ID "egg-tool-item-menu-id"
+
 enum {
   CREATE_MENU_PROXY,
   TOOLBAR_RECONFIGURED,
@@ -39,11 +42,14 @@ enum {
   PROP_0,
   PROP_VISIBLE_HORIZONTAL,
   PROP_VISIBLE_VERTICAL,
+  PROP_IS_IMPORTANT
 };
 
 static void egg_tool_item_init       (EggToolItem *toolitem);
 static void egg_tool_item_class_init (EggToolItemClass *class);
-
+static void egg_tool_item_finalize    (GObject *object);
+static void egg_tool_item_parent_set   (GtkWidget   *toolitem,
+				        GtkWidget   *parent);
 static void egg_tool_item_set_property (GObject         *object,
 					guint            prop_id,
 					const GValue    *value,
@@ -52,7 +58,6 @@ static void egg_tool_item_get_property (GObject         *object,
 					guint            prop_id,
 					GValue          *value,
 					GParamSpec      *pspec);
-
 static void egg_tool_item_realize       (GtkWidget      *widget);
 static void egg_tool_item_unrealize     (GtkWidget      *widget);
 static void egg_tool_item_map           (GtkWidget      *widget);
@@ -61,12 +66,12 @@ static void egg_tool_item_size_request  (GtkWidget      *widget,
 					 GtkRequisition *requisition);
 static void egg_tool_item_size_allocate (GtkWidget      *widget,
 					 GtkAllocation  *allocation);
-static void egg_tool_item_real_set_tooltip (EggToolItem *tool_item,
-					    GtkTooltips *tooltips,
-					    const gchar *tip_text,
-					    const gchar *tip_private);
+static gboolean egg_tool_item_real_set_tooltip (EggToolItem *tool_item,
+						GtkTooltips *tooltips,
+						const gchar *tip_text,
+						const gchar *tip_private);
 
-static GtkWidget *egg_tool_item_create_menu_proxy (EggToolItem *item);
+static gboolean egg_tool_item_create_menu_proxy (EggToolItem *item);
 
 
 static GObjectClass *parent_class = NULL;
@@ -101,18 +106,18 @@ egg_tool_item_get_type (void)
 }
 
 static gboolean
-create_proxy_accumulator (GSignalInvocationHint *hint,
-			  GValue *return_accumulator,
-			  const GValue *handler_return,
-			  gpointer user_data)
+egg_boolean_handled_accumulator (GSignalInvocationHint *ihint,
+				  GValue                *return_accu,
+				  const GValue          *handler_return,
+				  gpointer               dummy)
 {
-  GObject *proxy;
   gboolean continue_emission;
-
-  proxy = g_value_get_object (handler_return);
-  g_value_set_object (return_accumulator, proxy);
-  continue_emission = (proxy == NULL);
-
+  gboolean signal_handled;
+  
+  signal_handled = g_value_get_boolean (handler_return);
+  g_value_set_boolean (return_accu, signal_handled);
+  continue_emission = !signal_handled;
+  
   return continue_emission;
 }
 
@@ -128,6 +133,7 @@ egg_tool_item_class_init (EggToolItemClass *klass)
   
   object_class->set_property = egg_tool_item_set_property;
   object_class->get_property = egg_tool_item_get_property;
+  object_class->finalize = egg_tool_item_finalize;
 
   widget_class->realize       = egg_tool_item_realize;
   widget_class->unrealize     = egg_tool_item_unrealize;
@@ -135,6 +141,7 @@ egg_tool_item_class_init (EggToolItemClass *klass)
   widget_class->unmap         = egg_tool_item_unmap;
   widget_class->size_request  = egg_tool_item_size_request;
   widget_class->size_allocate = egg_tool_item_size_allocate;
+  widget_class->parent_set    = egg_tool_item_parent_set;
 
   klass->create_menu_proxy = egg_tool_item_create_menu_proxy;
   klass->set_tooltip       = egg_tool_item_real_set_tooltip;
@@ -153,15 +160,25 @@ egg_tool_item_class_init (EggToolItemClass *klass)
 							 _("Whether the toolbar item is visible when the toolbar is in a vertical orientation."),
 							 TRUE,
 							 G_PARAM_READWRITE));
-  
+
+  g_object_class_install_property (object_class,
+                                   PROP_IS_IMPORTANT,
+                                   g_param_spec_boolean ("is_important",
+                                                         _("Is important"),
+                                                         _("Whether the toolbar item is considered important. When TRUE, toolbar buttons show text in GTK_TOOLBAR_BOTH_HORIZ mode"),
+                                                         FALSE,
+                                                         G_PARAM_READWRITE));
+
   toolitem_signals[CREATE_MENU_PROXY] =
     g_signal_new ("create_menu_proxy",
 		  G_OBJECT_CLASS_TYPE (klass),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (EggToolItemClass, create_menu_proxy),
-		  create_proxy_accumulator, NULL,
-		  _egg_marshal_OBJECT__VOID,
-		  GTK_TYPE_WIDGET, 0);
+		  egg_boolean_handled_accumulator, NULL, /* FIXME: use gtk_boolean_handled() when
+							  * we are added to gtk+
+							  */
+		  _egg_marshal_BOOLEAN__VOID,
+		  G_TYPE_BOOLEAN, 0);
   toolitem_signals[TOOLBAR_RECONFIGURED] =
     g_signal_new ("toolbar_reconfigured",
 		  G_OBJECT_CLASS_TYPE (klass),
@@ -175,9 +192,11 @@ egg_tool_item_class_init (EggToolItemClass *klass)
 		  G_OBJECT_CLASS_TYPE (klass),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (EggToolItemClass, set_tooltip),
-		  NULL, NULL,
-		  _egg_marshal_VOID__OBJECT_STRING_STRING,
-		  G_TYPE_NONE, 3,
+		  egg_boolean_handled_accumulator, NULL, /* FIXME: use gtk_boolean_handled() when
+							  * we are added to gtk+
+							  */
+		  _egg_marshal_BOOLEAN__OBJECT_STRING_STRING,
+		  G_TYPE_BOOLEAN, 3,
 		  GTK_TYPE_TOOLTIPS,
 		  G_TYPE_STRING,
 		  G_TYPE_STRING);		  
@@ -191,13 +210,58 @@ egg_tool_item_init (EggToolItem *toolitem)
   toolitem->visible_horizontal = TRUE;
   toolitem->visible_vertical = TRUE;
   toolitem->homogeneous = FALSE;
+  toolitem->expand = FALSE;
 }
 
 static void
-egg_tool_item_set_property (GObject         *object,
-			    guint            prop_id,
-			    const GValue    *value,
-			    GParamSpec      *pspec)
+egg_tool_item_finalize (GObject *object)
+{
+  EggToolItem *item = EGG_TOOL_ITEM (object);
+
+  if (item->menu_item)
+    g_object_unref (item->menu_item);
+  
+  if (G_OBJECT_CLASS (parent_class)->finalize)
+    G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
+static void
+egg_tool_item_parent_set   (GtkWidget   *toolitem,
+			    GtkWidget   *prev_parent)
+{
+  egg_tool_item_toolbar_reconfigured (EGG_TOOL_ITEM (toolitem));
+}
+
+gboolean
+egg_tool_item_get_is_important (EggToolItem *tool_item)
+{
+  g_return_val_if_fail (EGG_IS_TOOL_ITEM (tool_item), FALSE);
+
+  return tool_item->is_important;
+}
+
+void
+egg_tool_item_set_is_important (EggToolItem *tool_item, gboolean is_important)
+{
+  g_return_if_fail (EGG_IS_TOOL_ITEM (tool_item));
+
+  is_important = is_important != FALSE;
+
+  if (is_important != tool_item->is_important)
+    {
+      tool_item->is_important = is_important;
+
+      gtk_widget_queue_resize (GTK_WIDGET (tool_item));
+
+      g_object_notify (G_OBJECT (tool_item), "is_important");
+    }
+}
+
+static void
+egg_tool_item_set_property (GObject      *object,
+			    guint         prop_id,
+			    const GValue *value,
+			    GParamSpec   *pspec)
 {
   EggToolItem *toolitem = EGG_TOOL_ITEM (object);
 
@@ -209,16 +273,19 @@ egg_tool_item_set_property (GObject         *object,
     case PROP_VISIBLE_VERTICAL:
       egg_tool_item_set_visible_horizontal (toolitem, g_value_get_boolean (value));
       break;
+    case PROP_IS_IMPORTANT:
+      egg_tool_item_set_is_important (toolitem, g_value_get_boolean (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
 }
 
 static void
-egg_tool_item_get_property (GObject         *object,
-			    guint            prop_id,
-			    GValue          *value,
-			    GParamSpec      *pspec)
+egg_tool_item_get_property (GObject    *object,
+			    guint       prop_id,
+			    GValue     *value,
+			    GParamSpec *pspec)
 {
   EggToolItem *toolitem = EGG_TOOL_ITEM (object);
 
@@ -230,6 +297,9 @@ egg_tool_item_get_property (GObject         *object,
     case PROP_VISIBLE_VERTICAL:
       g_value_set_boolean (value, toolitem->visible_vertical);
       break;
+    case PROP_IS_IMPORTANT:
+      g_value_set_boolean (value, toolitem->is_important);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -281,18 +351,25 @@ egg_tool_item_realize (GtkWidget *widget)
 }
 
 static void
-egg_tool_item_unrealize (GtkWidget *widget)
+destroy_drag_window (EggToolItem *toolitem)
 {
-  EggToolItem *toolitem;
-
-  toolitem = EGG_TOOL_ITEM (widget);
-
   if (toolitem->drag_window)
     {
       gdk_window_set_user_data (toolitem->drag_window, NULL);
       gdk_window_destroy (toolitem->drag_window);
       toolitem->drag_window = NULL;
     }
+}
+
+static void
+egg_tool_item_unrealize (GtkWidget *widget)
+{
+  EggToolItem *toolitem;
+
+  toolitem = EGG_TOOL_ITEM (widget);
+
+  destroy_drag_window (toolitem);
+  
   GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
 }
 
@@ -322,13 +399,22 @@ static void
 egg_tool_item_size_request (GtkWidget      *widget,
 			    GtkRequisition *requisition)
 {
-  GtkBin *bin = GTK_BIN (widget);
+  GtkWidget *child = GTK_BIN (widget)->child;
+  gint xthickness = widget->style->xthickness;
+  gint ythickness = widget->style->ythickness;
 
-  if (bin->child)
-    gtk_widget_size_request (bin->child, requisition);
+  if (child && GTK_WIDGET_VISIBLE (child))
+    {
+      gtk_widget_size_request (child, requisition);
+    }
+  else
+    {
+      requisition->height = 0;
+      requisition->width = 0;
+    }
   
-  requisition->width += GTK_CONTAINER (widget)->border_width * 2;
-  requisition->height += GTK_CONTAINER (widget)->border_width * 2;  
+  requisition->width += (xthickness + GTK_CONTAINER (widget)->border_width) * 2;
+  requisition->height += (ythickness + GTK_CONTAINER (widget)->border_width) * 2;  
 }
 
 static void
@@ -338,37 +424,47 @@ egg_tool_item_size_allocate (GtkWidget     *widget,
   EggToolItem *toolitem = EGG_TOOL_ITEM (widget);
   GtkAllocation child_allocation;
   gint border_width;
-  GtkWidget *child;
+  GtkWidget *child = GTK_BIN (widget)->child;
 
   widget->allocation = *allocation;
   border_width = GTK_CONTAINER (widget)->border_width;
 
-  if (toolitem->drag_window && GTK_WIDGET_REALIZED (widget))
+  if (toolitem->drag_window)
     gdk_window_move_resize (toolitem->drag_window,
                             widget->allocation.x + border_width,
                             widget->allocation.y + border_width,
                             widget->allocation.width - border_width * 2,
                             widget->allocation.height - border_width * 2);
   
-  child = GTK_BIN (toolitem)->child;
   if (child && GTK_WIDGET_VISIBLE (child))
     {
-      child_allocation.x = allocation->x + border_width;
-      child_allocation.y = allocation->y + border_width;
-      child_allocation.width = allocation->width - border_width * 2;
-      child_allocation.height = allocation->height - border_width * 2;
+      gint xthickness = widget->style->xthickness;
+      gint ythickness = widget->style->ythickness;
+      
+      child_allocation.x = allocation->x + border_width + xthickness;
+      child_allocation.y = allocation->y + border_width + ythickness;
+      child_allocation.width = allocation->width - 2 * (xthickness + border_width);
+      child_allocation.height = allocation->height - 2 * (ythickness + border_width);
       
       gtk_widget_size_allocate (child, &child_allocation);
     }
 }
 
-static GtkWidget *
+static gboolean
 egg_tool_item_create_menu_proxy (EggToolItem *item)
 {
-  if (GTK_BIN (item)->child)
-    return NULL;
-  else
-    return gtk_separator_menu_item_new ();
+  if (!GTK_BIN (item)->child)
+    {
+      GtkWidget *menu_item = NULL;
+
+      menu_item = gtk_separator_menu_item_new();
+
+      egg_tool_item_set_proxy_menu_item (item, MENU_ID, menu_item);
+
+      return TRUE;
+    }
+  
+  return FALSE;
 }
 
 EggToolItem *
@@ -463,7 +559,7 @@ egg_tool_item_set_expand (EggToolItem *tool_item,
 
 void
 egg_tool_item_set_pack_end (EggToolItem *tool_item,
-			    gboolean pack_end)
+			    gboolean     pack_end)
 {
   g_return_if_fail (EGG_IS_TOOL_ITEM (tool_item));
     
@@ -493,15 +589,20 @@ egg_tool_item_set_homogeneous (EggToolItem *tool_item,
     }
 }
 
-static void
+static gboolean
 egg_tool_item_real_set_tooltip (EggToolItem *tool_item,
 				GtkTooltips *tooltips,
 				const gchar *tip_text,
 				const gchar *tip_private)
 {
-  GtkBin *bin = GTK_BIN (tool_item);
+  GtkWidget *child = GTK_BIN (tool_item)->child;
 
-  gtk_tooltips_set_tip (tooltips, bin->child, tip_text, tip_private);
+  if (!child)
+    return FALSE;
+
+  gtk_tooltips_set_tip (tooltips, child, tip_text, tip_private);
+
+  return TRUE;
 }
 
 void
@@ -510,10 +611,12 @@ egg_tool_item_set_tooltip (EggToolItem *tool_item,
 			   const gchar *tip_text,
 			   const gchar *tip_private)
 {
+  gboolean retval;
+  
   g_return_if_fail (EGG_IS_TOOL_ITEM (tool_item));
 
   g_signal_emit (tool_item, toolitem_signals[SET_TOOLTIP], 0,
-		 tooltips, tip_text, tip_private);
+		 tooltips, tip_text, tip_private, &retval);
 }
 
 void
@@ -539,12 +642,7 @@ egg_tool_item_set_use_drag_window (EggToolItem *toolitem,
 	}
       else
 	{
-	  if (toolitem->drag_window)
-	    {
-	      gdk_window_set_user_data (toolitem->drag_window, NULL);
-	      gdk_window_destroy (toolitem->drag_window);
-	      toolitem->drag_window = NULL;
-	    }
+	  destroy_drag_window (toolitem);
 	}
     }
 }
@@ -600,3 +698,57 @@ egg_tool_item_get_visible_vertical (EggToolItem *toolitem)
 
   return toolitem->visible_vertical;
 }
+
+GtkWidget *
+egg_tool_item_retrieve_proxy_menu_item (EggToolItem *tool_item)
+{
+  gboolean retval;
+  
+  g_return_val_if_fail (EGG_IS_TOOL_ITEM (tool_item), NULL);
+
+  g_signal_emit (tool_item, toolitem_signals[CREATE_MENU_PROXY], 0, &retval);
+  
+  return tool_item->menu_item;
+}
+
+GtkWidget *
+egg_tool_item_get_proxy_menu_item (EggToolItem *tool_item,
+				   const gchar *menu_item_id)
+{
+  g_return_val_if_fail (EGG_IS_TOOL_ITEM (tool_item), NULL);
+  g_return_val_if_fail (menu_item_id != NULL, NULL);
+
+  if (tool_item->menu_item_id && strcmp (tool_item->menu_item_id, menu_item_id) == 0)
+    return tool_item->menu_item;
+
+  return NULL;
+}
+
+void
+egg_tool_item_set_proxy_menu_item (EggToolItem *tool_item,
+				   const gchar *menu_item_id,
+				   GtkWidget   *menu_item)
+{
+  g_return_if_fail (EGG_IS_TOOL_ITEM (tool_item));
+  g_return_if_fail (menu_item == NULL || GTK_IS_MENU_ITEM (menu_item));
+  g_return_if_fail (menu_item_id != NULL);
+
+  if (tool_item->menu_item_id)
+    g_free (tool_item->menu_item_id);
+      
+  tool_item->menu_item_id = g_strdup (menu_item_id);
+
+  if (tool_item->menu_item != menu_item)
+    {
+      if (tool_item->menu_item)
+	g_object_unref (G_OBJECT (tool_item->menu_item));
+      
+      if (menu_item)
+	{
+	  g_object_ref (menu_item);
+	  gtk_object_sink (GTK_OBJECT (menu_item));
+	}
+      
+      tool_item->menu_item = menu_item;
+    }
+}
