@@ -26,17 +26,15 @@
 #include "ephy-file-helpers.h"
 
 #include <string.h>
+#include <bonobo/bonobo-i18n.h>
 
 static EphyEmbedEvent *
 get_event_info (EphyWindow *window)
 {
 	EphyEmbedEvent *info;
-	EphyTab *tab;
 
-	tab = ephy_window_get_active_tab (window);
-	g_return_val_if_fail (tab != NULL, NULL);
-
-	info = ephy_tab_get_event (tab);
+	info = EPHY_EMBED_EVENT (g_object_get_data
+		(G_OBJECT (window), "context_event"));
 	g_return_val_if_fail (info != NULL, NULL);
 
 	return info;
@@ -58,6 +56,7 @@ popup_cmd_link_in_new_window (EggAction *action,
 
 	ephy_shell_new_tab (ephy_shell, NULL, tab,
 			    g_value_get_string (value),
+			    EPHY_NEW_TAB_OPEN_PAGE |
 			    EPHY_NEW_TAB_IN_NEW_WINDOW);
 }
 
@@ -77,6 +76,7 @@ popup_cmd_link_in_new_tab (EggAction *action,
 
 	ephy_shell_new_tab (ephy_shell, window, tab,
 			    g_value_get_string (value),
+			    EPHY_NEW_TAB_OPEN_PAGE |
 			    EPHY_NEW_TAB_IN_EXISTING_WINDOW);
 }
 
@@ -96,6 +96,7 @@ popup_cmd_image_in_new_tab (EggAction *action,
 
 	ephy_shell_new_tab (ephy_shell, window, tab,
 			    g_value_get_string (value),
+			    EPHY_NEW_TAB_OPEN_PAGE |
 			    EPHY_NEW_TAB_IN_EXISTING_WINDOW);
 }
 
@@ -115,12 +116,13 @@ popup_cmd_image_in_new_window (EggAction *action,
 
 	ephy_shell_new_tab (ephy_shell, NULL, tab,
 			    g_value_get_string (value),
+			    EPHY_NEW_TAB_OPEN_PAGE |
 			    EPHY_NEW_TAB_IN_NEW_WINDOW);
 }
 
 void
 popup_cmd_bookmark_link (EggAction *action,
-			     EphyWindow *window)
+			 EphyWindow *window)
 {
 	GtkWidget *new_bookmark;
 	EphyBookmarks *bookmarks;
@@ -130,6 +132,7 @@ popup_cmd_bookmark_link (EggAction *action,
 	const GValue *link_rel;
 	const GValue *link;
 	const GValue *link_is_smart;
+	const GValue *linktext;
 	const char *title;
 	const char *location;
 	const char *rel;
@@ -142,30 +145,37 @@ popup_cmd_bookmark_link (EggAction *action,
 	ephy_embed_event_get_property (info, "link", &link);
 	ephy_embed_event_get_property (info, "link_title", &link_title);
 	ephy_embed_event_get_property (info, "link_rel", &link_rel);
+	ephy_embed_event_get_property (info, "linktext", &linktext);
 
-	title = g_value_get_string (link_title);
 	location = g_value_get_string (link);
+	g_return_if_fail (location);
+
 	rel = g_value_get_string (link_rel);
 	is_smart = g_value_get_int (link_is_smart);
 
-	g_return_if_fail (location);
+	title = g_value_get_string (link_title);
 
-	if (!title || !title[0])
+	if (title == NULL || title[0] == '\0')
+	{
+		title = g_value_get_string (linktext);
+	}
+
+	if (title == NULL || title[0] == '\0')
 	{
 		title = location;
 	}
 
 	bookmarks = ephy_shell_get_bookmarks (ephy_shell);
-	new_bookmark = ephy_new_bookmark_new
-		(bookmarks, GTK_WINDOW (window), location);
-	ephy_new_bookmark_set_title
-		(EPHY_NEW_BOOKMARK (new_bookmark), title);
-	ephy_new_bookmark_set_smarturl
-		(EPHY_NEW_BOOKMARK (new_bookmark), rel);
-	g_signal_connect (G_OBJECT (new_bookmark), "response",
-			  G_CALLBACK (ephy_new_bookmark_response_cb),
-			  NULL);
-	gtk_widget_show (new_bookmark);
+	if (ephy_new_bookmark_is_unique (bookmarks, GTK_WINDOW (window),
+					 location))
+	{
+		new_bookmark = ephy_new_bookmark_new
+			(bookmarks, GTK_WINDOW (window),
+			 is_smart ? rel : location);
+		ephy_new_bookmark_set_title
+			(EPHY_NEW_BOOKMARK (new_bookmark), title);
+		gtk_widget_show (new_bookmark);
+	}
 }
 
 void
@@ -184,6 +194,7 @@ popup_cmd_frame_in_new_tab (EggAction *action,
 
 	ephy_shell_new_tab (ephy_shell, window, tab,
 			    location,
+			    EPHY_NEW_TAB_OPEN_PAGE |
 			    EPHY_NEW_TAB_IN_EXISTING_WINDOW);
 
 	g_free (location);
@@ -205,6 +216,7 @@ popup_cmd_frame_in_new_window (EggAction *action,
 
 	ephy_shell_new_tab (ephy_shell, NULL, tab,
 			    location,
+			    EPHY_NEW_TAB_OPEN_PAGE |
 			    EPHY_NEW_TAB_IN_NEW_WINDOW);
 
 	g_free (location);
@@ -220,43 +232,33 @@ popup_cmd_copy_to_clipboard (EphyWindow *window, const char *text)
 }
 
 void
-popup_cmd_copy_email (EggAction *action,
-                      EphyWindow *window)
+popup_cmd_copy_link_address (EggAction *action,
+			     EphyWindow *window)
 {
-	EphyEmbedEvent *info;
-	const char *location;
+	EphyEmbedEvent *event;
+	const char *address;
 	const GValue *value;
-	EphyEmbed *embed;
 
-	embed = ephy_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
+	event = get_event_info (window);
+	g_return_if_fail (IS_EPHY_EMBED_EVENT (event));
 
-	info = get_event_info (window);
-	ephy_embed_event_get_property (info, "email", &value);
-	location = g_value_get_string (value);
-	popup_cmd_copy_to_clipboard (window, location);
-}
-
-void
-popup_cmd_copy_link_location (EggAction *action,
-			      EphyWindow *window)
-{
-	EphyEmbedEvent *info;
-	const char *location;
-	const GValue *value;
-	EphyEmbed *embed;
-
-	embed = ephy_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-
-	info = get_event_info (window);
-	ephy_embed_event_get_property (info, "link", &value);
-	location = g_value_get_string (value);
-	popup_cmd_copy_to_clipboard (window, location);
+	if (event->context & EMBED_CONTEXT_EMAIL_LINK)
+	{
+		ephy_embed_event_get_property (event, "email", &value);
+		address = g_value_get_string (value);
+		popup_cmd_copy_to_clipboard (window, address);
+	}
+	else if (event->context & EMBED_CONTEXT_LINK)
+	{
+		ephy_embed_event_get_property (event, "link", &value);
+		address = g_value_get_string (value);
+		popup_cmd_copy_to_clipboard (window, address);
+	}
 }
 
 static void
 save_property_url (EggAction *action,
+		   const char *title,
 		   EphyWindow *window,
 		   gboolean ask_dest,
 		   gboolean show_progress,
@@ -288,11 +290,9 @@ save_property_url (EggAction *action,
 					      EMBED_PERSIST_SHOW_PROGRESS);
 	}
 
-	ephy_embed_utils_save (GTK_WIDGET (window),
+	ephy_embed_utils_save (GTK_WIDGET (window), title,
 			       CONF_STATE_DOWNLOADING_DIR,
-			       ask_dest,
-                               FALSE,
-                               persist);
+			       ask_dest, FALSE, persist);
 
 	g_object_unref (G_OBJECT(persist));
 }
@@ -320,9 +320,9 @@ void
 popup_cmd_download_link (EggAction *action,
 			 EphyWindow *window)
 {
-	save_property_url (action, window,
+	save_property_url (action, _("Download link"), window,
 		           eel_gconf_get_boolean
-		           (CONF_STATE_DOWNLOADING_DIR),
+		           (CONF_ASK_DOWNLOAD_DEST),
 		           TRUE, "link");
 }
 
@@ -330,7 +330,8 @@ void
 popup_cmd_save_image_as (EggAction *action,
 			 EphyWindow *window)
 {
-	save_property_url (action, window, TRUE, FALSE, "image");
+	save_property_url (action, _("Save Image As"),
+			   window, TRUE, FALSE, "image");
 }
 
 #define CONF_DESKTOP_BG_PICTURE "/desktop/gnome/background/picture_filename"
@@ -417,7 +418,8 @@ void
 popup_cmd_save_background_as (EggAction *action,
 			      EphyWindow *window)
 {
-	save_property_url (action, window, TRUE, FALSE, "background_image");
+	save_property_url (action, _("Save Background As"),
+			   window, TRUE, FALSE, "background_image");
 }
 
 void
@@ -433,6 +435,8 @@ popup_cmd_open_frame (EggAction *action,
 	ephy_embed_get_location (embed, FALSE, &location);
 
 	ephy_embed_load_url (embed, location);
+
+	g_free (location);
 }
 
 void
@@ -453,4 +457,3 @@ popup_cmd_open_image (EggAction *action,
 
 	ephy_embed_load_url (embed, location);
 }
-
